"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _helperPluginUtils = require("@babel/helper-plugin-utils");

var _pluginSyntaxNullishCoalescingOperator = require("@babel/plugin-syntax-nullish-coalescing-operator");

var _core = require("@babel/core");

// Heavily inspired by https://github.com/babel/babel/blob/main/packages/babel-plugin-proposal-nullish-coalescing-operator/src/index.js
var createVisitor = function createVisitor(identifier) {
  return {
    LogicalExpression: function LogicalExpression(path) {
      var node = path.node,
          scope = path.scope;

      if (node.operator !== '??' || !_core.types.isIdentifier(node.right) || node.right.name !== identifier) {
        return;
      }

      var ref = scope.maybeGenerateMemoised(node.left);
      var assignment; // skip creating extra reference when `left` is static

      if (ref === null) {
        ref = node.left;
        assignment = _core.types.cloneNode(node.left);
      } else {
        assignment = _core.types.assignmentExpression('=', ref, node.left);
      }

      path.replaceWith(_core.types.logicalExpression('&&', _core.types.binaryExpression('!==', assignment, _core.types.nullLiteral()), _core.types.binaryExpression('!==', _core.types.cloneNode(ref), scope.buildUndefinedNode())));
    }
  };
};

var _default = (0, _helperPluginUtils.declare)(function (api, _ref) {
  var _ref$identifier = _ref.identifier,
      identifier = _ref$identifier === void 0 ? 'EXISTS' : _ref$identifier;
  api.assertVersion(7);
  return {
    name: 'transform-nullish-operator',
    inherits: _pluginSyntaxNullishCoalescingOperator.syntaxNullishCoalescingOperator,
    visitor: {
      // We could use `LogicalExpression` directly here, unfortunately,
      // `instanbul-lib-instrument` adds branch coverage collection using `Program.enter` when it gets loaded
      // and thus misunderstand `?? EXISTS` as an actual elvis operator,
      // so we need to compile it out ASAP!
      Program: {
        enter: function enter(path) {
          path.traverse(createVisitor(identifier));
        }
      }
    }
  };
});

exports["default"] = _default;